/*! gsheets 2.0.0-alpha.3 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.gsheets = global.gsheets || {})));
}(this, (function (exports) { 'use strict';

var BASE_URL = 'https://spreadsheets.google.com/feeds/';

// Utility functions

// Get value from a nested structure or null.
function getIn(o, keys) {
  var k = keys[0],
      ks = keys.slice(1);
  if (!o.hasOwnProperty(k)) return null;
  return ks.length ? getIn(o[k], ks) : o[k];
}

// Fetching

function fetchFeed(params) {
  var url = BASE_URL + params.join('/') + '/public/values?alt=json';
  return fetch(url).then(function (response) {
    return response.json();
  }).catch(function (err) {
    return Promise.reject(new Error('Could not parse response. Make sure the spreadsheet does exist.'));
  }).then(function (data) {
    return new Promise(function (resolve, reject) {
      if (data.feed) {
        resolve(data.feed);
      } else {
        reject(new Error('No feed was returned'));
      }
    });
  });
}

function fetchSpreadsheet(key) {
  return fetchFeed(['worksheets', key]).then(parseSpreadsheetFeed);
}

function fetchWorksheetById(key, worksheetId) {
  return fetchFeed(['cells', key, worksheetId]).then(parseWorksheetFeed);
}

function fetchWorksheetByTitle(key, worksheetTitle) {
  return fetchSpreadsheet(key).then(function (spreadsheet) {
    return new Promise(function (resolve, reject) {
      var worksheet = spreadsheet.worksheets ? spreadsheet.worksheets.filter(function (d) {
        return d.title === worksheetTitle;
      })[0] : null;
      if (worksheet) {
        resolve(fetchWorksheetById(key, worksheet.id));
      } else {
        reject(new Error('No worksheet with title \'' + worksheetTitle + '\' found.'));
      }
    });
  });
}

// Parsing

function parseWorksheetIdInSpreadsheetFeed(uri) {
  var re = /.*\/(.+)$/,
      matches = re.exec(uri);
  return matches ? matches[1] : null;
}

function parseWorksheet(worksheet) {
  return {
    id: parseWorksheetIdInSpreadsheetFeed(getIn(worksheet, ['id', '$t'])),
    title: getIn(worksheet, ['title', '$t'])
  };
}

function parseSpreadsheetFeed(feed) {
  return {
    updated: getIn(feed, ['updated', '$t']),
    title: getIn(feed, ['title', '$t']),
    worksheets: feed.entry ? feed.entry.map(parseWorksheet) : null
  };
}

function parseWorksheetFeed(feed) {
  return {
    updated: getIn(feed, ['updated', '$t']),
    title: getIn(feed, ['title', '$t']),
    data: feed.entry ? parseCellsIntoRows(feed.entry) : null
  };
}

function getCellData(cell) {
  var data = cell.gs$cell;
  return {
    col: +data.col,
    row: +data.row,
    value: data.numericValue ? +data.numericValue : data.$t
  };
}

function createRowFromHeaders(headers) {
  return headers.reduce(function (row, key) {
    row[key] = null;
    return row;
  }, {});
}

function parseCellsIntoRows(cells) {
  var cellsData = cells.map(getCellData),
      headerCells = cellsData.filter(function (d) {
    return d.row === 1;
  }),
      headers = headerCells.map(function (d) {
    return d.value;
  }),
      headersByCol = headerCells.reduce(function (byCol, d) {
    byCol[d.col] = d.value;
    return byCol;
  }, {}),
      bodyCells = cellsData.filter(function (d) {
    return d.row !== 1;
  }),
      bodyByRow = bodyCells.reduce(function (byRow, d) {
    var row = byRow[d.row] || createRowFromHeaders(headers),
        key = headersByCol[d.col];
    row[key] = d.value;
    byRow[d.row] = row;
    return byRow;
  }, {});

  return Object.keys(bodyByRow).sort(function (a, b) {
    return +a - +b;
  }).map(function (row) {
    return bodyByRow[row];
  });
}

// Public API

var getWorksheet = fetchWorksheetByTitle;
var getWorksheetById = fetchWorksheetById;
var getSpreadsheet = fetchSpreadsheet;

exports.getWorksheet = getWorksheet;
exports.getWorksheetById = getWorksheetById;
exports.getSpreadsheet = getSpreadsheet;

Object.defineProperty(exports, '__esModule', { value: true });

})));
